<?php

namespace Modules\St\Http\Controllers;

use App\Models\User;
use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Hash;
use Modules\Hr\Entities\Employee;
use Modules\St\Entities\Role;
use Modules\St\Entities\Suppliers;
use Modules\St\Entities\UserRole;


class UserController extends Controller
{
    /**
     * Display a listing of the resource.
     * @return Renderable
     */
    public function index()
    {
        return view('st::index');
    }

    /**
     * Show the form for creating a new resource.
     * @return Renderable
     */
    public function create()
    {
        return view('st::create');
    }

    /**
     * Store a newly created resource in storage.
     * @param Request $request
     * @return Renderable
     */
    public function store(Request $request)
    {
        //
    }

    /**
     * Show the specified resource.
     * @param int $id
     * @return Renderable
     */
    public function show($id)
    {
        return view('st::show');
    }

    /**
     * Show the form for editing the specified resource.
     * @param int $id
     * @return Renderable
     */
    public function edit($id)
    {
        return view('st::edit');
    }

    /**
     * Update the specified resource in storage.
     * @param Request $request
     * @param int $id
     * @return Renderable
     */
    public function update(Request $request, $id)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     * @param int $id
     * @return Renderable
     */
    public function destroy($id)
    {
        //
    }



    public function allUserRole()
    {
        try {
            $userRoles =  Role::all();
            $responseBody = $this->responseBody(true, "UserRole", "all", $userRoles);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "UserRole", "error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }



    public function allEmployee()
    {
        try {
            $employees = Employee::all();

            $employee_arr = [];
            for ($i = 0; $i < count($employees); $i++) {
                array_push($employee_arr, ["img" => $employees[$i]['photo_parth'], "id" => $employees[$i]['employee_id'], "value" => $employees[$i]['name_withinitial']]);
            }
            $responseBody = $this->responseBody(true, "Employee", "all", $employee_arr);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "Employee", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }





    public function allSupplier()
    {
        try {
            $supplier = DB::table('st_suppliers')->select('*')->get();

            $supplier_arr = [];
            foreach ($supplier as $sup) {
                array_push($supplier_arr, ["img" => "", "id" => $sup->supplier_id, "value" => $sup->supplier_name]);
            }
            $responseBody = $this->responseBody(true, "Supplier", "all", $supplier_arr);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "Supplier", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }




    public function allCustomer()
    {
        try {
            $customer = DB::table('st_customers')->select('*')->get();

            $customer_arr = [];
            foreach ($customer as $cus) {
                array_push($customer_arr, ["img" => "", "id" => $cus->customer_id, "value" => $cus->customer_name]);
            }
            $responseBody = $this->responseBody(true, "Customer", "all", $customer_arr);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "Customer", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }




    public function saveUser(Request $request)
    {
        try {
            $user = new User();
            $user->name = $request->get('username');
            $user->email = $request->get('email');
            $user->password = Hash::make($request->get('password'));
            $user->user_id = $request->get('user_id');
            $user->user_type = $request->get('userType');
            $save = $user->save();

            if ($save) {
                $user_role = new UserRole();
                $user_role->user_id = $user->id;
                $user_role->role_id = $request->get('userRoll');
                $user_role->save();
            }


            $responseBody = $this->responseBody(true, "User", "saved", $request->get('userRoll'));
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "User", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }



    public function allUser()
    {
        try {
            $users =  DB::select('SELECT
            roles.`id` AS role_id,
            roles.`name` AS role_name,
            users.`id` AS user_id,
            users.`name` AS user_name,
            users.`user_type` AS user_type,
            users.`email` AS email
       FROM
            `roles` roles INNER JOIN `user_roles` user_roles ON roles.`id` = user_roles.`role_id`
            INNER JOIN `users` users ON user_roles.`user_id` = users.`id`');

            $responseBody = $this->responseBody(true, "User", "all", $users);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "User", "error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }




    public function getUser($user_id, $role_id)
    {

        $user = User::find($user_id);
        $role = Role::find($role_id);
        $type = NULL;
        if ($user->user_type == "Employee") {
            $type = Employee::find($user->user_id);
        } else if ($user->user_type == "Supplier") {
            $type = Suppliers::find($user->user_id);
        }
        return response()->json(["data" =>  $this->responseBody(true, "User", "getUser", ["user" => $user, "role" => $role, "type" => $type])]);
    }



    public function updateUser(Request $request, $id)
    {
        try {
            $user = User::find($id);
            $user->name = $request->get('username');
            $user->email = $request->get('email');
            if ($request->get('password') != NULL) {
                $user->password = Hash::make($request->get('password'));
            }
            $user->user_id = $request->get('user_id');
            $user->user_type = $request->get('type');
            $update = $user->save();

            if ($update) {
                DB::update('UPDATE `user_roles` SET role_id = "' . $request->get('userRoll') . '" WHERE user_id = "' . $id . '"');
            }


            $responseBody = $this->responseBody(true, "User", "updated", $request->get('type'));
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "User", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }



    public function deleteUser($id)
    {
        try {
            $user = User::find($id);
            $status = $user->delete();
            $responseBody = $this->responseBody(true, "User", "deleted", $status);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "User", "error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }


    /**
     * responseBody
     * This is used to create response.
     * @param success This is the paramter require boolean
     * @param name This is the paramter require ui table name
     * @param message This is the paramter require message content
     * @param result This is the paramter require result as some of data to return client
     * @return Json This returns as response.
     */
    private function responseBody($success, $name, $message, $result)
    {
        $body = [
            "success" => $success,
            "message" => $message,
            "name" => $name,
            "result" => $result
        ];
        return $body;
    }
}
